@@ -96,10 +96,10 @@ RcppExport SEXP Rcurvature( SEXP vb_, SEXP it_)
 			      Rcpp::Named("K2") = K2
 			      );
   } catch (std::exception& e) {
-    ::Rf_error( e.what());
+    forward_exception_to_r( e );
     return wrap(1);
   } catch (...) {
     ::Rf_error("unknown exception");
-  }
+  } return R_NilValue; // -Wall
 
 }
